        program ldss2
C
C   Dear UK/NL/ES site manager
C   
C   Below is a new edition of SIGNAL.FOR, the Fortran code which can
C   be used to predict signal-to-noise ratios for observations made
C   with ING instruments.  Please make available to observers at your
C   site.
C   
C   Thanks
C   
C   Chris Benn
C
C
C
C   SIGNAL in [CRB.LPDATA]
C
C   Chris Benn  La Palma 
C
C   Calculates throughput and signal-to-noise for spectroscopic and 
C   direct-imaging observations carried out with ING instruments.
C   This version is written in VAX Fortran.  If you have any 
C   problems compiling it on your machine, please let me know.
C   Compile command for a SUN believed to be:
C     f77 signal.for -o signal.exe -e
C     If you compile it on a Vax using version earlier than 6.0
C     you will need fort /CONTIN=99 to overcome the default value
C     of 19 for number of continuation lines
C
C     After using ftp to extract the .EXE file from the web
C     chmod +x signal.exe allows you to run it
C
C   1990 Nov 26  Version 1.0
C   1991 Jun 11  Version 2.0
C   1992 Feb  8  Corrected erroneous I, Z CCD sensitivities
C   1992 Mar 12  Added detector QE for Tek CCD (version 3.0)
C   1992 Mar 25  Updated GEC/EEV CCD responses with PRJ's latest data
C                (version 3.1)
C   1992 Apr  3  Added LDSS spectroscopic and imaging 
C                Generalising code to be instrument-independent (version 4.0)
C   1993 Oct 14  Updating UES efficiencies using Max Pettini's data
C   1993 Dec  9  Adding INT FORD and GEC7 QE 
C   1994 Sep  6  Version 6.0: adding WHT prime (instr 19), removing JKT QUBES
C                and grism options; adding WHT WYFFOS and hitchhiker;
C                updating readout noises and default CCD;
C                correcting IPCS2 pixel size to 10 microns;
C                adding derotator eff 0.75 for UES;
C                added R1200B grating (assume eff as R600B)
C   1995 Jan 23  Added TRUETHRU empirical/predicted data (+ extrapolations)
C                WHT FOS throughput based on 1/1/92 measurement
C                Version 7.0
C   1995 Mar 24  Removed incorrect warning that no throughput data for WYFFOS
C                Added correct DISP for WYFFOS
C   1995 May 10  Removed tabs.  Version 7.1
C   2000 Jul  1  Converted for LDSS2@Magellan - IRS  
C
C
C   The program calculates the number of photons/A from the object
C   (SOBJ) and photons/A/arcsec**2 from the sky (SSKY) as =
C      10**(-mag/2.5)                 where mag is MOBJ or MSKYDATA(IBAND)
C                                     MOBJ is magnitude for a point object,
C                                     mag/arcsec**2 for spectroscopy of 
C                                     extended object
C     *TIME                           integration time in sec
C     *KDATA(IBAND)*10000             conversion mag -> 
C                                     photons/m**2/sec/A
C     *TELAREA(ITEL(IINSTR))          telescope area (m**2)
C     *MIRROR**IMIRROR(IINSTR)        efficiency of main mirrors
C     *EFFINSTR(IBAND,IINSTR)         throughput of instrument
C     *EFFGRAT(IBAND,IGRAT,IINSTR)    efficiency of grating
C     *EFFDET(IBAND,IDET)             q. efficiency of detector
C     *TRUETHRU(IBAND,IINSTR)         ratio empirical/theoretical throughput,
C                                     now determined for most instruments
C                                     (see SPECTRUM March 1995), values 
C                                     typically 0.7 - 1.1, other values 
C                                     extrapolated or set to 1.0 (all were
C                                     1.0 for pre-1995 versions of SIGNAL).
C
C   For imaging, the number of photons from object (SOBJ2) and sky/pixel (SSKY2)
C   are then calculated as:
C      SOBJ2=SOBJ*BAND(IBAND)
C      SSKY2=SSKY*BAND(IBAND)*ARCPIX*ARCPIX
C
C   For spectroscopy, for a point object, the program gives the number of
C   photons/A (SOBJ), the number of photons per pixel-step in wavelength
C   (SOBJ2) and number of photons/pixel from the sky (SSKY2):
C      SOBJ2=SOBJ*LAMPIX
C      SSKY2=SSKY*SLIT*ARCPIX*LAMPIX
C
C   For spectroscopy, for an extended object, the program gives the number
C   of photons/A/arcsec**2 of object (SOBJ), the number of photons/pixel
C   from the object (SOBJ2) and the number of photons/pixel from the sky
C   (SSKY2, calculated as for a point source):
C      SOBJ2=SOBJ*SLIT*ARCPIC*LAMPIX
C
C   The programn also calculates the scales at the detector in arcsec/pixel
C   (ARCPIX) and A/pixel (LAMPIX) as:
C      ARCPIX=ARCSCALE(IINSTR)  *PIXEL(IDET)
C     LAMPIX=DISP(IGRAT,IINSTR)*PIXEL(IDET)
C
C   See the lines in the subroutine HELP at the end of the code for
C   caveats about individual instruments
C   
C   Central wavelengths and bandwidths assumed:
C      U  3600 A  (3300 - 3800 A) (glass, broad-band)
C      B  4400 A  (4000 - 5000 A) (KPNO)
C      V  5500 A  (5000 - 6000 A) (KPNO)
C      R  7000 A  (6000 - 7200 A) (KPNO)
C      I  8200 A  (7200 - 9200 A) (KPNO)
C      Z  9500 A  (8500 - 9700 A) (glass, broad-band)
C
C   Description of variables:
C     ADU       electrons per recorded count (DETECTOR,INSTRUMENT)
C     ANS, ANS2 1-char ans
C     ARCPIX    "/pixel at detector
C     ARCSCALE  "/mm scale at detector for each INSTR
C     AREA      INSTR * relevant efficiencies (i.e. effective coll. area)
C     BAND      eff. bandwidths (includes effect of throughput)
C     CBAND, CINSTR, CGRATING, CDET - names of bands, 
C               instr. combinations, gratings, detectors
C               CGRATING is (IGRAT,IINSTR)
C     CONTROL   control file band names
C     DISP      dispersion A/mm for each grating (IGRAT,IINSTR)
C     EFFDET    detector efficiencies for each band    (BAND,DETECTOR)
C               For GEC, sensitivities are for coated CCD
C     EFFGRAT   grating efficiencies for each band (BAND,GRATING,INSTRUMENT)
C     EFFINSTR  throughput of instrument for each band (BAND,INSTRUMENT)
C     FWHM      FWHM arcsec (seeing + object)
C     IBAND, IINSTR, IGRAT, IDET - band, instrument, grating and detector numbers
C     ICONTROL  3 integers - instrument, grating, detector
C     IMIRROR   number of mirrors between sky and instrument
C     INSTR     eff. coll. area of telescope * wavelength indep. part of
C               instr. throughput (in m**2)
C     ITEL      telescope number for the given IINSTR
C                 1 = JKT     2 = INT     3 = WHT     (4 = AAT     5 = UKIRT)
C     ITRANS    determines order in which instrument menu written
C     ITYPE     observation type for the given IINSTR
C                 1 = imaging, 2 = spectroscopic
C     IUNIT     input unit number (5 unless MODE = 1, in which case IUNIT=1)
C     JYFLUX    Jy flux (1E-26 W/Hz/m**2) for mag=0 in each band (Z is approx)
C     KDATA     photons/s/sq.cm/A for mag = 0 for each band
C     LAMPIX    A/pixel at detector
C     LOOP      main loop counter
C     MIRROR    reflective efficiency assumed for mirror
C     MODE      mode: 0 => interactive
C                     1 => reads from SIGNAL.DAT, writes to OUTPUT.LIS
C     MOBJ      magnitude of object
C               (per sq arcsec for spectroscopy with FWHM=1)
C     MSKYDATA  sky brightness in mag/arcsec**2
C     NBAND, NINSTR, NGRATING(IINSTR), NDET - numbers of bands, 
C               instruments, gratings and detectors
C     NCONTROL1, NCONTROL2 - range of entries in SIGNAL.DAT to use in
C               non-interactive mode
C     PIXEL     pixel size (mm) for each detector
C     READ      readout noise (electrons)
C     SEE       3.14159/4 * (FWHM/PIXSIZE)**2 i.e. area over which image
C               spread out
C     SLIT, SLIT2 - slit-width (arcsec)
C     SNR       signal-to-noise
C     SOBJ      number of photons/A from object
C               (per sq arcsec for spectroscopy with FWHM = 1)
C     SOBJ2     For spectroscopy, photons/pixel (sky or ext object with FWHM=1)
C               or photons/pixel step in wavelength i.e. after integrating
C               in the spatial direction (FWHM = 0)
C     SSKY      number of photons/arcsec**2/A from sky
C     SSKY2     SSKY per pixel
C     TELAREA   geometric collecting area of telescope ITEL(IINSTR)
C     TIME      integration time (sec)
C     TRUETHRU  empirical/predicted throughputs (BAND,INSTR)
C     TRUE0     value of TRUETHRU passed to SIGNAL_DISPLAY
C     VERSION   Version number and date
C
C     Variables for passing to SIGNAL_DISPLAY:
C     QINST,QBAND,QGRAT,QGRAT2,QDET,GEOMAREA,THROUGHPUT,PIXSIZE,DETEFF,PHOTONS,
C     READOUT,BWIDTH
C
      implicit none
      integer i,j,k,ninstr,ndet,iband,iinstr,igrat,itrans(20),
     1  itel(20),itype(20),imirror(20),ngrating(20)
      integer idet,nband,loop,mode,iunit,icontrol(3),
     1 ncontrol1,ncontrol2,jj
      real read(10),slit,telarea(50),effinstr(6,20),
     1 effdet(6,10),mskydata(6),effgrat(6,20,20),disp(20,20)
      real mobj,time,band(6),jyflux(6),arcscale(20)
      real arcpix,lampix,mirror,area,kdata(6),sobj,ssky
      real pixel(10),sky,sobj2,ssky2,see,slit2,adu(10,20),true0
      real fwhm,geomarea,throughput,pixsize,deteff,photons,bwidth,
     1  readout,
     1    truethru(6,20)
      character*6 cgrating(20,20),cdet(10),qgrat,qdet,qgrat2
      character*1 cband(6),ans,qband,ans2,control(6)
      character*25 cinstr(20),qinst
      character*28 version

C
C   Version number and date
C
      version='Version 7.1      10-MAY-1995'

C
C   Character data
C
      data cband/'U','B','V','R','I','Z'/
      data cinstr/'WHT ISIS                 ',
     1           'WHT FOS                  ',
     1           'WHT TAURUS2 imaging f/2  ',
     1           'WHT TAURUS2 imaging f/4  ',
     1           'WHT TAURUS2 + etalon f/2 ',
     1           'WHT TAURUS2 + etalon f/4 ',
     1           'WHT UES                  ',
     1           'WHT aux-port imaging     ',
     1           'INT prime imaging        ',
     1           'INT IDS 235 camera       ',
     1           'INT IDS 500 camera       ',
     1           'INT FOS                  ',
     1           'JKT imaging              ',
     1           'JKT RBS                  ',
     1           'JKT QUBES                ',
     1           'WHT WYFFOS               ',
     1           'LDSS2 imaging            ',
     1           'LDSS2 spectroscopy       ',
     1           'WHT prime imaging        ',
     1           'WHT hitchhiker imaging   '/
      data cgrating/
     1           'R158R ','R316R ','R600R ','R1200R',  ! WHT ISIS
     1           'R158B ','R300B ','R600B ','R1200B','H2400B',
     1           11*'      ',
     1           '1stORD','2ndORD',18*'      ',        ! WHT FOS
     1           20*'      ',                          ! WHT TAU
     1           20*'      ',                          ! WHT TAU
     1           20*'      ',                          ! WHT TAU
     1           20*'      ',                          ! WHT TAU
     1           20*'      ',                          ! WHT UES
     1           20*'      ',                          ! WHT aux-port
     1           20*'      ',                          ! INT prime
     1           'R150V ','R300V ','R400B ',
     1           'R400V ','R400R ','R600R ',           !IDS
     1           'R600IR','R632V ','R831R ','R900V ','R1200U',
     1           'R1200B','R1200Y','R1200R','H1800V','H2400B',
     1           4*'      ',                           ! IDS
     1           'R150V ','R300V ','R400B ',
     1           'R400V ','R400R ','R600R ',           
     1           'R600IR','R632V ','R831R ','R900V ','R1200U',
     1           'R1200B','R1200Y','R1200R','H1800V','H2400B',
     1           4*'      ',                           ! IDS
     1           '1stORD','2ndORD',18*'      ',        ! INT FOS
     1           20*'      ',                          ! JKT imaging
     1           ' 300  ',' 600  ','1200  ','2400  ',16*'      ',  ! RBS
     1           20*'      ',                          ! JKT QUBES
     1           'R158R ','R316R ','R600R ','R1200R',  ! WHT WYFFOS (i.e.
     1           'R158B ','R300B ','R600B ','R1200B','H2400B', ! copy ISIS)
     1           11*'      ',
     1           20*'      ',                          ! LDSS imaging
     1           'Low   ','MedBlu','MedRed','High  ',16*'      ',  ! LDSS
     1           20*'      ',                          ! 
     1           20*'      '/                          !

C
C   Bandpass and La Palma sky data from INT prime-focus guide
C
c      data mskydata/21.4,22.3,21.4,20.4,19.3,19.2/
      data mskydata/22.0,23.0,21.9,21.2,20.2,19.8/
c
c   using La Silla values from ESO manual (Z-value extrapolated)
c   for Las Campanas
c
      data kdata/783.0,1463.0,1050.0,873.0,376.0,329.0/
      data band/260.,780.,675.,1000.,1400.,1200./
C
C   Mag -> Jy conversions from Johnson H.L., 1966. ARAA 4, 193
C
      data jyflux/1880.,4440.,3810.,3010.,2430.,2200./
C
C   Instrumental data, mainly from Observers' Guide November 1988
C   UES efficiency is 0.86 (cross disperser) * 0.7 (camera) * 0.8 (etalon) 
C   = 0.48 * 0.75 (derotator) = 0.36
C   TAURUS throughput=0.6
C   INT prime efficiency is 0.98**6 for 6 surfaces of corrector
C   IDS throughput = 0.45*0.9
C
      data read/60.,6.,4.,0.,0.,7.2,12.,9.,3.1,0./
      data adu/
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,2.0,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,1.5,1.4,0.8,0.0,
     1 3.8,1.1,1.1,0.,0.,2.7,0.0,0.0,0.8,0.0/
c
c---bodged pixel scale for LDSS2@Magellan 24.6/1.55 -> 15.9"/mm
c
      data arcscale/14.9,35.0,23.25,12.4,23.25,12.4,15.1,4.51,
     1             24.7,29.4,13.8,58.0,13.8,49.47,0.,0.,
     1             15.9,15.9,17.5,0./
      data pixel/0.030,0.022,0.022,0.015,0.01,0.024,0.015,
     1             0.022,0.024,0./
c
c---added new detector (#9) - SITe#2 24um 2048^2 for Magellan
c
      data effinstr/
C                     U    B    V    R    I    Z
     1           0.45,0.45,0.45,0.45,0.45,0.45,  ! WHT ISIS
     1           0.30,0.50,0.35,0.70,0.60,0.00,  ! WHT FOS
     1           0.60,0.60,0.60,0.60,0.60,0.60,  ! WHT TAURUS IMAGING
     1           0.60,0.60,0.60,0.60,0.60,0.60,  ! WHT TAURUS IMAGING
     1           0.48,0.48,0.48,0.48,0.48,0.48,  ! WHT TAURUS ETALON
     1           0.48,0.48,0.48,0.48,0.48,0.48,  ! WHT TAURUS ETALON
     1           0.36,0.36,0.36,0.36,0.36,0.36,  ! WHT UES
     1           1.00,1.00,1.00,1.00,1.00,1.00,  ! WHT aux-port imaging
     1           0.89,0.89,0.89,0.89,0.89,0.89,  ! INT prime imaging
     1           0.41,0.41,0.41,0.41,0.41,0.41,  ! INT IDS 235 camera
     1           0.41,0.41,0.41,0.41,0.41,0.41,  ! INT IDS 500 camera
     1           0.30,0.60,0.30,0.60,0.70,0.00,  ! INT FOS
     1           1.00,1.00,1.00,1.00,1.00,1.00,  ! JKT imaging
     1           1.00,1.00,1.00,1.00,1.00,1.00,  ! JKT RBS
     1           1.00,1.00,1.00,1.00,1.00,1.00,  ! JKT QUBES
     1           0.36,0.36,0.36,0.36,0.36,0.36,  ! WHT WYFFOS
     1           0.20,0.60,0.75,0.70,0.40,0.10,  ! LDSS imaging
     1           0.20,0.60,0.75,0.70,0.40,0.10,  ! LDSS + grating
     1           1.00,1.00,1.00,1.00,1.00,1.00,  ! WHT prime imaging
     1           0.00,1.00,1.00,1.00,1.00,0.00/  ! WHT hitchhiker
      data cdet  /'RCA   ','GEC   ','EEV   ','IPCS1 ','IPCS2 ',
     1           'TEK#5 ','FORD  ','GEC7  ','SITe#2','      '/
      data effdet /0.15,0.60,0.70,0.70,0.55,0.20,
     1         0.13,0.14,0.32,0.45,0.35,0.12,
     1         0.13,0.14,0.32,0.45,0.35,0.12,
     1         0.12,0.11,0.06,0.03,0.00,0.00,
     1         0.12,0.11,0.06,0.03,0.00,0.00,
     1         0.60,0.75,0.77,0.74,0.59,0.19,
     1         0.10,0.20,0.33,0.36,0.28,0.13,
     1         0.28,0.40,0.67,0.76,0.61,0.33,
     1         0.48,0.52,0.55,0.57,0.37,0.10,
     1         0.00,0.00,0.00,0.00,0.00,0.00/
      data effgrat/
     1          0.0,0.0,0.65,0.7,0.6,0.0,   ! WHT ISIS 158R
     1          0.0,0.0,0.75,0.7,0.45,0.0,  ! 316R
     1          0.0,0.0,0.55,0.60,0.5,0.0,  ! 600R
     1          0.0,0.0,0.45,0.5,0.5,0.0,   ! 1200R
     1          0.7,0.55,0.4,0.35,0.0,0.0,   ! 158B
     1          0.7,0.65,0.55,0.42,0.0,0.0,  ! 300B
     1          0.65,0.6,0.5,0.38,0.0,0.0,   ! 600B
     1          0.5,0.55,0.6,0.38,0.0,0.0,   ! 1200B added 13/10/94
     1          0.2,0.4,0.3,0.0,0.0,0.0,66*0.0, ! 2400B
     1          120*1.0,                    ! WHT FOS
     1          120*1.0,                    ! WHT TAURUS imaging
     1          120*1.0,                    ! WHT TAURUS imaging
     1          120*1.0,                    ! WHT TAURUS + etalon
     1          120*1.0,                    ! WHT TAURUS + etalon
     1          120*0.55,                   ! WHT UES
     1          120*1.0,                    ! WHT aux-port imaging
     1          120*1.0,                    ! INT prime
     1         0.4,0.55,0.65,0.55,0.0,0.0,  !IDS 235
     1         0.4,0.55,0.7,0.65,0.4,0.0,
     1         0.55,0.45,0.3,0.0,0.0,0.0,
     1         0.4,0.65,0.65,0.5,0.35,0.0,
     1         0.0,0.0,0.4,0.55,0.55,0.55,
     1         0.0,0.0,0.55,0.6,0.55,0.5,
     1         0.0,0.0,0.0,0.3,0.65,0.7,
     1         0.4,0.6,0.65,0.6,0.45,0.4,
     1         0.0,0.0,0.4,0.55,0.65,0.65,
     1         0.0,0.65,0.65,0.55,0.0,0.0,
     1         0.7,0.6,0.0,0.0,0.0,0.0,
     1         0.65,0.6,0.5,0.4,0.0,0.0,
     1         0.35,0.55,0.65,0.65,0.4,0.4,
     1         0.0,0.0,0.4,0.6,0.65,0.0,
     1         0.0,0.3,0.45,0.5,0.0,0.0,
     1         0.25,0.55,0.5,0.0,0.0,0.0,24*0.0,
     1         0.4,0.55,0.65,0.55,0.0,0.0,  ! IDS 500
     1         0.4,0.55,0.7,0.65,0.4,0.0,
     1         0.55,0.45,0.3,0.0,0.0,0.0,
     1         0.4,0.65,0.65,0.5,0.35,0.0,
     1         0.0,0.0,0.4,0.55,0.55,0.55,
     1         0.0,0.0,0.55,0.6,0.55,0.5,
     1         0.0,0.0,0.0,0.3,0.65,0.7,
     1         0.4,0.6,0.65,0.6,0.45,0.4,
     1         0.0,0.0,0.4,0.55,0.65,0.65,
     1         0.0,0.65,0.65,0.55,0.0,0.0,
     1         0.7,0.6,0.0,0.0,0.0,0.0,
     1         0.65,0.6,0.5,0.4,0.0,0.0,
     1         0.35,0.55,0.65,0.65,0.4,0.4,
     1         0.0,0.0,0.4,0.6,0.65,0.0,
     1         0.0,0.3,0.45,0.5,0.0,0.0,
     1         0.25,0.55,0.5,0.0,0.0,0.0,24*0.0,
     1          120*1.0,                    ! INT FOS
     1          120*1.0,                    ! JKT imaging
     1         1.,1.,1.,1.,1.,1.,           ! WHTRBS
     1         1.,1.,1.,1.,1.,1.,
     1         1.,1.,1.,1.,1.,1.,
     1         1.,1.,1.,1.,1.,1.,96*0.0,
     1          120*1.0,                    ! JKT QUBES
     1          0.0,0.0,0.65,0.7,0.6,0.0,   ! WYFFOS (copy of ISIS data)
     1          0.0,0.0,0.75,0.7,0.45,0.0,
     1          0.0,0.0,0.55,0.60,0.5,0.0,
     1          0.0,0.0,0.45,0.5,0.5,0.0,
     1          0.7,0.55,0.4,0.0,0.0,0.0,
     1          0.7,0.65,0.55,0.0,0.0,0.0,
     1          0.65,0.6,0.5,0.0,0.0,0.0,
     1          0.65,0.6,0.5,0.0,0.0,0.0,
     1          0.2,0.4,0.3,0.0,0.0,0.0,66*0.0,
     1          120*1.0,                          ! WHT imaging
     1          0.2,0.6,0.8,0.7,0.6,0.5,          ! WHT LDSS low
     1          0.4,0.7,0.75,0.65,0.4,0.3,        ! WHT LDSS blue 
     1          0.0,0.25,0.7,0.75,0.6,0.5,        ! WHT LDSS red
     1          0.35,0.55,0.6,0.3,0.0,0.0,96*0.0, ! WHT LDSS high
     1          120*1.0,
     1          120*1.0/
      data disp/
     1           121.,62.,33.,17.,120.,64.,33.,17.,8.,11*0.0, ! WHT ISIS
     1           395.,195.,18*0.0,                            ! WHT FOS
     1           20*0.0,                                      ! WHT TAURUS
     1           20*0.0,                                      ! WHT TAURUS
     1           20*0.0,                                      ! WHT TAURUS
     1           20*0.0,                                      ! WHT TAURUS
     1           2.5,2.5,18*0.0,                              ! WHT UES
     1           20*0.0,                                      ! WHT aux-port
     1           20*0.0,                                      ! INT imaging
     1          271.3,138.5,104.5,104.5,104.4,69.8,70.2,66.5, ! IDS235
     1           50.7, 46.4, 35.3, 35.3, 35.2,34.8,23.2,17.5,4*0.0,
     1          132.2, 66.1, 49.6, 49.7, 49.9,33.3,33.4,31.6, ! IDS500
     1           23.9, 22.1, 16.7, 16.7, 16.4,15.5,10.5, 8.1,4*0.0,
     1           486.,245.,18*0.0,                            ! INT FOS
     1           20*0.0,                                      ! JKT imaging
     1           160.,80.,40.,19.,16*0.0,                     ! JKT RBS
     1           1.8,1.5,1.28,1.12,1.0,0.9,0.82,0.75,0.7,11*0.0,! JKT QUBES
     1           479.,245.,131.,67.,479.,254.,131.,67.,33.,11*0.0,! WHT WYFFOS
     1           20*0.0,                                      ! WHT LDSS im.
     1           470.,220.,220.,100.,16*0.0,                  ! WHT LDSS
     1           20*0.0,                                      ! WHT prime
     1           20*0.0/                                      ! Hitchhiker
      data itrans/1,2,17,18,16,3,4,5,6,7,8,19,20,9,10,11,12,13,
     1              14,15/
c
c  added - Magellan itel=4 - area assumes 1.2m hole - 
c  LDSS2 at Nasmyth (imirror=3) 
c
      data telarea/0.66,4.41,12.47,32.0,46*0.0/
      data itel     /3,3,3,3,3,3,3,3,2, 2, 2,2,1,1,1,3,4,4,3,3/
      data imirror  /2,2,2,2,2,2,3,3,1, 2, 2,2,2,2,2,1,3,3,1,3/
      data itype    /2,2,1,1,2,2,2,1,1, 2, 2,2,1,2,2,2,1,2,1,1/
      data ngrating /9,2,0,0,0,0,2,0,0,16,16,2,0,4,0,9,0,4,0,0/
C
C   TRUETHRU(BAND,INSTR) are from table of 7/9/94 in file, with extrapolations
C   where values unknown
C
      data truethru/
C                     U   B   V   R   I   Z
     1            0.9,0.6,0.7,0.8,0.4,0.7, ! ISIS
     1            0.8,0.8,0.8,0.7,0.8,0.8, ! WHT FOS
     1            0.8,0.7,1.0,0.7,0.9,0.8, ! TAURUS imaging f/2
     1            0.8,0.7,1.0,0.7,0.9,0.8, ! TAURUS imaging f/4
     1            0.8,0.7,1.0,0.7,0.9,0.8, ! TAURUS etalon f/2
     1            0.8,0.7,1.0,0.7,0.9,0.8, ! TAURUS etalon f/4
     1            0.5,0.5,0.6,0.5,0.6,0.5, ! UES
     1            0.6,0.6,0.6,0.6,0.6,0.6, ! aux-port
     1            0.9,0.8,0.8,0.6,0.9,0.7, ! INT prime
     1            0.4,0.5,0.5,0.6,0.6,0.6, ! IDS 235
     1            0.4,0.5,0.5,0.6,0.6,0.6, ! IDS 500
     1            0.3,0.5,0.6,0.8,0.8,0.8, ! INT FOS
     1            0.9,0.7,0.9,0.9,1.0,1.1, ! JKT imaging
     1            1.0,1.0,1.0,1.0,1.0,1.0, ! JKT RBS
     1            1.0,1.0,1.0,1.0,1.0,1.0, ! JKT QUBES
     1            1.0,1.0,1.0,1.0,1.0,1.0, ! WYFFOS
     1            1.0,1.0,1.0,1.0,1.0,1.0, ! LDSS imaging
     1            1.7,0.7,0.7,0.7,0.7,0.7, ! LDSS spectroscopy
     1            0.7,0.4,0.8,0.6,1.0,0.8, ! WHT prime
     1            1.0,0.5,0.8,0.5,0.5,1.0/ ! Hitchhiker

      mirror=0.85

      nband=6
      ninstr=20
      ndet=8

C
C   Set default values of parameters
C
      iinstr=17
      iband=3
      idet=9
      igrat=1
      qgrat='Low   '
      fwhm=1
      mobj=24
      slit=1
      time=3600

C
C   MODE = 0 - normal use
C          1 - set with undocumented menu option X to read controls from file
C
      mode=0

C
C   Input unit = terminal unless MODE = 1
C   If MODE = 1, unit 5 becomes JUNK.LIS to which contents of SIGNAL.DAT
C   are rewritten
C
      iunit=5

C
C   Welcome lines
C
c      call signal_welcome(version)
      print *,' '
      print *,'LDSS2@Magellan S/N Calculator IRS 07/01/00'

C
C   Main menu loop
C
      loop=0      
1      continue
      loop=loop+1
      print *,' '
      if(loop.gt.1.and.mode.eq.0) print *,
     1 'Type I G D B M T F S or K to change a parameter'
      if(mode.eq.0) print *,
     1 'Type return to continue,',
     1 ' Q to quit, H for more information'
      read(iunit,'(a1)') ans
      if(mode.eq.0) print *,' '

C
C   Stop
C
      if(ans.eq.'Q'.or.ans.eq.'q') goto 500

C
C   Automatic (MODE = 1)
C
      if(ans.eq.'X'.or.ans.eq.'x') then
      mode=1
      iunit=5
      open(unit=1,name='SIGNAL.DAT',status='old')
      open(unit=iunit,name='JUNK.LIS',status='new')
C     1      carriagecontrol='list')
C
C   Rewrite SIGNAL.DAT to JUNK.LIS in required format
C
      print *,'Range of entries in SIGNAL.DAT to use?'
      print *,'(0 0 -> first to last)'
      read(*,*) ncontrol1,ncontrol2
      if(ncontrol1.eq.0) ncontrol1=1
      if(ncontrol2.eq.0) ncontrol2=1000000
      do i=1,ncontrol2
      read(1,*,end=223) icontrol
      read(1,'(6a1)') control
      if(i.lt.ncontrol1) goto 888
      do j=1,6
      if(control(j).ne.' ') then
            if(icontrol(1).eq.1.or.icontrol(1).eq.10.or.
     1         icontrol(1).eq.11.or.icontrol(1).eq.14) then
            write(iunit,224) icontrol,control(j)
224     format('I',/,i3,/,'G',/,i3,/,'D',/,i3,/,'B',/,a1,/,' ')
            else
            write(iunit,225) icontrol,control(j)
225     format('I',/,i3,/,'G',i3,/,'D',/,i3,/,'B',/,a1,/,' ')
            endif
      endif
      enddo
888      continue
      enddo
223      continue
      write(iunit,'(''Q'')')
      rewind(unit=iunit)

      open(unit=7,name='OUTPUT.LIS',status='new')
C     1      carriagecontrol='list')
      endif

C
C   Type rudimentary help
C
      if(ans.eq.'H'.or.ans.eq.'h') then
      call helpsub
      goto 1
      endif

C
C   Get band
C
      if(ans.eq.'B'.or.ans.eq.'b') then
      if(mode.eq.0) then
      print *,
     1       'Approximate throughput for U, B, V, R, I or Z bands?'
c      print *,
c     1       '(NB for imaging in B, KPNO B filter assumed (Harris has',
c     1      ' efficiency 0.7 * KPNO)'
      endif
      read(iunit,'(a1)') ans2
      if(ans2.eq.'U'.or.ans2.eq.'u') iband=1
      if(ans2.eq.'B'.or.ans2.eq.'b') iband=2
      if(ans2.eq.'V'.or.ans2.eq.'v') iband=3
      if(ans2.eq.'R'.or.ans2.eq.'r') iband=4
      if(ans2.eq.'I'.or.ans2.eq.'i') iband=5
      if(ans2.eq.'Z'.or.ans2.eq.'z') iband=6
      endif

C
C   Get apparent magnitude
C
992   if(ans.eq.'M'.or.ans.eq.'m') then
      if(mode.eq.0) write(6,104) cband(iband)
104      format(' Apparent magnitude in band ',a1,'?')
      if(mode.eq.0) print *,
     1 '(negative number will be interpreted as Jy (1E-26 W/Hz/m**2))'

      read(iunit,*,err=992) mobj
      if(mobj.lt.0) mobj=-2.5*alog10(-mobj/jyflux(iband))
      endif

C
C   Get telescope/instrument combination
C      
      if(ans.eq.'I'.or.ans.eq.'i') then
999   if(mode.eq.0) print *,'Telescope/instrument?' 
      do k=3,4
      i=itrans(k)
      if(mode.eq.0) write(6,101) k-2,cinstr(i)
101      format(i3,2x,a25)
      enddo
      if(mode.eq.0) write(6,102)
102      format(' Choose a number')
      read(iunit,*,err=999) k
      if((k.lt.0).or.(k.gt.2)) goto 999
      k=k+2
      iinstr=itrans(k)
      if(iinstr.eq.2.or.iinstr.eq.12) then
      idet=2
      print *,'Changing detector to GEC'
      endif
      if(itype(iinstr).eq.1) then
      fwhm=1
      else
      fwhm=0
      endif

C
C   Reset QGRAT
C
      if(igrat.ne.0) then
      qgrat=cgrating(igrat,iinstr)
      endif

      endif

C
C   Get slit width
C
      if(ans.eq.'S'.or.ans.eq.'s') then
998   if(mode.eq.0) print *,'Slit width (arcsec)?'
      read(iunit,*,err=998) slit
      endif

C
C   Get sky brightness
C
997   if(ans.eq.'K'.or.ans.eq.'k') then
      if(mode.eq.0) then
       print *,'Current value of sky brightness in ',
     1 cband(iband),' = ',mskydata(iband),
     1 ' mags/arcsec**2'
       print *,
     1'SIGNAL''s initial settings are for La Palma dark sky'
       print *,
     1'For moon, brighten sky by M mags as follows:'
       print *,
     1' Phase:            New  Crescent  Quarter  Gibbous  Full'
       print *,
     1' Phase:             0.0     0.25     0.5       0.75   1.0'
       print *,
     1' Approx day:        1       4        8        12     15'
       print *,
     1' B, G or D:         D       G        G         B      B'
       print *,
     1' Illum. frac. %     0      25       50        75    100'
       print *,
     1' M (U, B, V)        0                1         3      4'
       print *,
     1' M (R)              0                0.8       1.9    2.9'
       print *,
     1' M (I)              0                0.4       1.0    1.7'
       print *,
     1' '
       print *,
     1'These figures are interpolated from results obtained by'
       print *,
     1'Derek Jones in 1989, using the JKT'
       print *,
     1' '
       print *,
     1'Note that the quarter moon (i.e. half disc illuminated)'
       print *,
     1'is a factor of 10 (not 2) fainter than full, due to'
       print *,
     1'the opposition effect (also responsible for gegenschein'
       print *,
     1'on ecliptic and dry heiligenschein on Earth)'
       print *,
     1' '
       print *,
     1'New sky brightness for ',cband(iband),' band?'
      endif
      read(iunit,*,err=997) mskydata(iband)
      endif

C
C   If NGRATING(IINSTR) > 0, get grating
C
      if(ans.eq.'G'.or.ans.eq.'g') then
996   if(mode.eq.0) print *,'Grating?'
      if(ngrating(iinstr).gt.0) then
      do i=1,ngrating(iinstr)
      if(mode.eq.0) write(6,105) i,cgrating(i,iinstr),
     1      effgrat(iband,i,iinstr)
105      format(i3,2x,a6,' approx. efficiency in this band = ',f4.2)
      enddo
      if(mode.eq.0) write(6,102)
      read(iunit,*,err=996) igrat
      if(igrat.gt.0) then
            qgrat=cgrating(igrat,iinstr)
      else
            qgrat='      '
      endif
      endif
      endif

C
C   Get detector
C
995   if(ans.eq.'D'.or.ans.eq.'d') then
      if(mode.eq.0) then
      print *,'Detector?'
c      print *,'  TEK, EEV in use all telecsopes and foci'
c      print *,'  TEK is assumed to be at 185 K'
c      print *,'  Other CCDs, see latest issue of SPECTRUM'
c      print *,'  IPCS for UES, TAURUS or ISIS blue arm only'
      endif
      do jj=1,2
	if(jj.eq.1) i=6
	if(jj.eq.2) i=9
      if(mode.eq.0) write(6,106) jj,cdet(i),effdet(iband,i)
106      format(i3,2x,a6,' approx. efficiency in this band = ',f4.2)
      enddo
      if(mode.eq.0) write(6,102)
      read(iunit,*,err=995) jj
	if(jj.eq.1)then
	  idet=6
	else
	  idet=9
	endif
      endif

C
C   Get exposure time
C
994   if(ans.eq.'T'.or.ans.eq.'t') then
      if(mode.eq.0) print *,'Exposure time (sec)?'
      read(iunit,*,err=994) time
      endif

C
C   Get FWHM
C
993   if(ans.eq.'F'.or.ans.eq.'f') then
      if(itype(iinstr).eq.1) then
      if(mode.eq.0) print *,'FWHM object * seeing?'
      else
      if(mode.eq.0) 
     1    print *,'Point (0) or extended (1) source'
      endif      
      read(iunit,*,err=993) fwhm
      endif

C
C   Calculate effective collecting area for TEL * INSTR * DET
C
      geomarea=telarea(itel(iinstr))
      area=geomarea*mirror**(float(imirror(iinstr)))
     1 *effinstr(iband,iinstr)*effgrat(iband,igrat,iinstr)

      throughput=area/geomarea
      if(area.eq.0.and.mode.eq.0) then
      print *,
     1 '*** WARNING - throughput = 0 (if spectroscopic, check grating)'
      print *,' '
      endif

      area=area*effdet(iband,idet)      


C
C   Calculate scales at detector in arcsec/pixel and Angstrom/pixel
C
      arcpix=arcscale(iinstr)*pixel(idet)
      lampix=disp(igrat,iinstr)*pixel(idet)

C
C   Calculate number of photons from object and from sky/arcsec**2
C   per Angstrom
C
      sobj=kdata(iband)*10000*area*time*10.**(-mobj/2.5)
      ssky=
     1     kdata(iband)*10000*area*time*10.**(-mskydata(iband)/2.5)

C
C   Correct for empirical/predicted throughput ratio
C
      sobj=sobj*truethru(iband,iinstr)
      ssky=ssky*truethru(iband,iinstr)


C
C   Display parameter settings to user
C
      qinst=cinstr(iinstr)
      qband=cband(iband)
      qdet=cdet(idet)
      qgrat2=qgrat
      sky=mskydata(iband)
      deteff=effdet(iband,idet)
      pixsize=pixel(idet)
      photons=kdata(iband)
      bwidth=band(iband)
      readout=read(idet)
      slit2=slit
      true0=truethru(iband,iinstr)

      if(ngrating(iinstr).gt.0) then      
      qgrat2=qgrat
      else
      qgrat2=' '
      endif

      if(itype(iinstr).eq.1) then
      slit2=0
      else
      bwidth=0
      endif

      call signal_display(mode,ans,iinstr,itype,
     1  qinst,qband,qgrat2,qdet,fwhm,mobj,slit2,
     1  sky,time,geomarea,throughput,deteff,pixsize,photons,
     1  bwidth,readout,true0)

C
C   Direct imaging calculation and output
C
      if(itype(iinstr).eq.1) then
      if(mode.eq.0) print *,' '
      sobj2=sobj*band(iband)
      ssky2=ssky*band(iband)*arcpix*arcpix
      if(mode.eq.0) then
      print *,'Detected photons       from object = ',sobj2
      print *,'Detected photons/pixel from sky    = ',ssky2
      endif
      if(mode.eq.1.and.ans.eq.' ') write(7,141) sobj2,ssky2
141      format(f20.5)
      if(arcpix.gt.0) then
      see=3.14159*(fwhm/arcpix)**2/4.
      if(sobj2.gt.0) then
      sobj2=sobj2/sqrt(sobj2+see*(ssky2+read(idet)*read(idet)))
      if(mode.eq.0) print *,'Signal-to-noise           = ',sobj2
      if(mode.eq.1.and.ans.eq.' ') write(7,141) sobj2
      else
      if(mode.eq.1.and.ans.eq.' ') write(7,141) 0.0
      endif
      endif
      else

C
C   Spectroscopic calculation and output
C
      if(fwhm.eq.0) then
      if(mode.eq.0) 
     1   print *,'Detected photons per A from object = ',sobj
      else
      if(mode.eq.0) print *,
     1 'Detected photons per A from each sq arcsec of object = ',sobj
      endif
      if(mode.eq.1.and.ans.eq.' ') write(7,141) sobj
      
      if(fwhm.eq.0) then
      sobj2=sobj*lampix
      else
      sobj2=sobj*slit*arcpix*lampix
      endif

      if(sobj2.gt.0) then
      if(fwhm.eq.0) then
      if(mode.eq.0) print *,
     1       'Detected photons/pixel-step-in-wavelenth from object = ',
     1       sobj2
      else
      if(mode.eq.0) print *,
     1      'Detected photons/pixel from object = ',sobj2
      endif
      if(mode.eq.1.and.ans.eq.' ') write(7,141) sobj2
      else
      if(mode.eq.1.and.ans.eq.' ') write(7,141) 0.0
      endif
      ssky2=ssky*slit*arcpix*lampix
      if(ssky2.ne.0) then
      if(mode.eq.0) 
     1  print *,'Detected photons/pixel from sky    = ',ssky2
      if(mode.eq.1.and.ans.eq.' ') write(7,141) ssky2
      else
      if(mode.eq.1.and.ans.eq.' ') write(7,141) 0.0
      endif
      endif

C
C   Angular scale at detector
C
      if(mode.eq.0) print *,' '
      if(arcpix.ne.0) then
      if(mode.eq.0) write(6,111) arcpix
      if(mode.eq.1.and.ans.eq.' ') write(7,141) arcpix
      else
      if(mode.eq.1.and.ans.eq.' ') write(7,141) 0.0
      endif
111      format(' Angular scale at detector = ',f7.2,' "/pixel')

C
C   Dispersion at detector
C
      if(lampix.ne.0) then
      if(mode.eq.0) write(6,113) lampix
      if(mode.eq.1.and.ans.eq.' ') write(7,141) lampix
      else
      if(mode.eq.1.and.ans.eq.' ') write(7,141) 0.0
      endif
113      format(' Dispersion    at detector = ',f8.3,' A/pixel')


C
C   Photons/ADU conversion
C
      if(idet.eq.1.or.idet.eq.2.or.idet.eq.3.or.idet.eq.6.
     1 or.idet.eq.7.or.idet.eq.8) then
      if(mode.eq.0) write(6,151) adu(idet,iinstr)
151      format(' For this CCD, 1 count     = ',f6.1,
     1      '   detected photons approx.')
      endif

      if(mode.eq.0) print *,' '
      if(idet.eq.5.and.mode.eq.0) print *,
     1 'IPCS2 throughput asumed equal to that of IPCS1'
      if(iinstr.eq.14.or.iinstr.eq.15.
     1 and.mode.eq.0) print *,
     1 '*** WARNING - no throughput data for instrument - 1.0 assumed'
      goto 1

500      continue
      end





      subroutine signal_display(mode,ans,iinstr,itype,
     1  qinst,qband,qgrat,qdet,fwhm,mobj,slit,
     1  sky,time,geomarea,throughput,deteff,pixsize,photons,
     1  bwidth,readout,true0)
C
C   Displays parameter settings to user
C
      implicit none
      integer i,mode,iinstr,itype(20)
      real fwhm,mobj,sky,time,geomarea,throughput,deteff,pixsize,
     1 photons,bwidth,readout,slit,true0
      character*6 qgrat,qdet
      character*37 text(20)
      character*25 qinst
      character*1 qband,ans

      do i=1,20
      text(i)=' '
      enddo

      print *,' '
      text(1)='G Grating                      '//qgrat
      text(2)='D Detector                     '//qdet
      text(3)='B Band                         '//qband

C
C   Text strings which depend on whether imaging or spectroscopic
C
      if(itype(iinstr).eq.1) then
C
C   Imaging
C
      write(text(4),'(
     1    ''M Apparent magnitude          '',f6.1)') mobj
      write(text(6),'(
     1    ''F FWHM (object*seeing)        '',f6.1)') fwhm
      write(text(15),'(
     1    ''  Effective bandwidth (A)   '',f9.2)') bwidth
      else
C
C   Spectroscopic
C
      text(15)=' '
      write(text(6),'(
     1    ''F Point (0) or extended (1)   '',f6.1)') fwhm
      if(fwhm.eq.0) then
      write(text(4),'(
     1    ''M Apparent magnitude          '',f6.1)') mobj
      else
      write(text(4),'(
     1    ''M Apparent magnitude/sq.arcsec'',f6.1)') mobj
      endif
      endif

C
C   Remaining text strings
C
      write(text(5),'(
     1    ''T Integration time (sec)   '',f9.1)') time
      write(text(7),'(
     1    ''S Slit width (arcsec)         '',f6.1)') slit
      write(text(8),'(
     1    ''  Tel. geometric area (sq.m)   '',f6.2)') geomarea
      write(text(9),'(
     1    ''  Telescope/instr. throughput  '',f6.2)') throughput
      write(text(10),'(
     1    ''  Detector efficiency          '',f6.2)') deteff
      write(text(11),'(
     1    ''  Pixel size (microns)        '',f6.1)') pixsize*1000
      write(text(12),'(
     1    ''  Readout noise (electrons)   '',f6.1)') readout
      write(text(13),'(
     1    ''  Photons/s/A/sq.cm mag=0    '',i5)')  int(photons)
      write(text(14),'(
     1    ''K Sky brightness mag/sq.arcsec '',f6.2)') sky
      write(text(15),'(
     1    ''  Empirical/theoretical        '',f6.2)') true0

      if(mode.eq.0) 
     1  write(6,101) qinst,text(8),(text(i),text(i+8),i=1,7)
101   format(' I Instrument = ',a25,1x,a37,/,7(1x,a37,4x,a37,/),
     1 40x,a37)
      if(mode.eq.1.and.ans.eq.' ') 
     1  write(7,109) iinstr,qinst,qgrat,qband,qdet
109   format(1x,i5,' = INSTR',/,1x,a25,/,1x,a6,/,1x,a1,/,1x,a6)

      end





      subroutine signal_welcome(version)
      implicit none
      character*28 version

      print *,' '
      print *,'SIGNAL              '//version
     1  //'       CRB @ AST-STAR.CAM.AC.UK'
      print *,' '
      print *,
     1 'For spectroscopic observations of point sources, the program'
      print *,
     1 'estimates the number of photons/Angstrom and the number of'
      print *,
     1 'photons/pixel-step-in-wavelength detected for a given'
      print *,
     1 'combination of telescope, instrument, wavelength band,'
      print *,
     1 'apparent magnitude and exposure time.  Accuracy +- 30 percent.'
      print *,
     1 'Detected photons/Angstrom ='
      print *,' 10.**(-mag/2.5) * photons/sec/A/m**2 for mag = 0'
      print *,
     1 '                 * geometric area (mirror less obstruction)'
      print *,
     1 '                 * efficiency of telescope (mirrors and lenses)'
      print *,'                 * collimator efficiency',
     1 '(where appropriate)'
      print *,
     1 '                 * grating/etalon efficiency',
     1 ' (where appropriate)' 
      print *,'                 * camera efficiency',
     1' (where appropriate)'
      print *,'                 * detector efficiency'
      print *,'                 * exposure time (sec)'
      print *,
     1 'A similar calculation is carried out for extended sources.'
      print *,' '
      print *,
     1 'For direct imaging, the program estimates the number of ',
     1 'object and'
      print *,
     1 'sky photons detected, and the signal-to-noise.'

      end






      subroutine helpsub
C
C   Types rudimentary help
C
      implicit none

      print *,' '
      print *,' Spectroscopic estimates require that you calculate',
     1' source magnitude in slit (so including slit losses)'
      print *,' '
      print *,' Imaging estimate based on detection aperture of ',
     1' FWHM-diameter - to be reliable this should be 2*expected seeing'
      print *,' '
      print *,' Empirical/theoretical: estimate of relative efficiency',
     1' from measurements on WHT - may not be relevant for Magellan'
      print *,' '
      print *,
     1'These are some of the things that SIGNAL doesn''t take',
     1' into account:'
      print *,' Throughput of atmosphere (0.9 in V - see',
     1 ' La Palma Technical Note 31)'
      print *,' Vignetting at large field radius'
      print *,
     1  ' Colour, ND and polarisation filters in light path (but'
      print *,
     1  '  BEWARE contamination by second order if no order-sorting'
      print *,'  filter used)'
c      print *,' ISIS dichroic (efficiency about 0.85)'
      print *,
     1  ' Vignetting by slit (use program LIGHT_IN_SLIT (RGOLUN76))'
      print *,' Fall in grating efficiency at large angles'
c      print *,' ISIS polarisation optics throughput, about 0.9'
      print *,
     1  ' Dark current, cosmic rays, non-linearity of CCD response'
      print *,
     1  ' Brightening of sky due to moonlight (up to 7 mag)'
      print *,
     1  ' Brightening of sky at solar maximum (about 1 mag?)'
      print *,' '
c      print *,
c     1  ' Note that for FOS, second-order gives U, B, first V,R,I'
      end

